from django.contrib import admin
from django.contrib import messages
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.forms.models import ModelMultipleChoiceField, ModelChoiceField
from django.utils.translation import ugettext_lazy as _
from django import forms
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.html import escape
from django.contrib.auth.models import User
from django.conf.urls.defaults import patterns, url
from django.conf import settings
from django.utils.translation import ugettext as ugt
from django.utils.encoding import force_unicode
from django.utils.formats import date_format
from django.views.decorators.csrf import csrf_protect
from django.utils.decorators import method_decorator
from django.db import transaction
from django.contrib.admin.templatetags.admin_list import _boolean_icon
from django.contrib.admin.options import IncorrectLookupParameters
from django.db.models import Q

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin, SettingsAdmin
from jumbo.apps.autocomplete import autocomplete
from jumbo.apps.autocomplete import AutocompleteModelChoiceField
from jumbo.forms.fields import JumboDateField, ChainedChoiceField
from jumbo.forms.models import AdminModelForm
from jumbo.core.urlresolvers import reverse
from jumbo.middleware.thread_local import get_request, get_current_user
from jumbo.apps.project.models import Project

from ticket.models import Settings, Ticket, CategoryTranslation, Category, TicketUser, TicketDocuments
from ticket.models import TipologyTranslation, Tipology, TicketProject, TicketAssistance
from ticket.models import STATUS_CODES, PRIORITY_CODES
from ticket.email import *
from ticket import views as ticket_views

from organization.models import Organization

# COLLEGO I VARI SEGNALI PER INVIARE LE MAIL
import ticket.signals
import datetime

admin.site.register(Settings, SettingsAdmin)

csrf_protect_m = method_decorator(csrf_protect)

#########################################
#              TICKET                   #
#########################################
class TicketDocumentInline(admin.TabularInline):
    model = TicketDocuments
    extra = 0
    fields = ('file', 'description',)


class TicketAdminSearchForm(forms.ModelForm):
    class Meta:
        model = Ticket
        display_fields = (
            'id__in', ('title__icontains','description__icontains'),
            ('project__name__icontains', 'organization__name__icontains',),
            ('status', 'category'),'keywords__icontains',
            ('date_create__gte', 'date_create__lte'),
            ('deadline__gte', 'deadline__lte')
        )

    def __init__(self, *args, **kwargs):
        super(TicketAdminSearchForm, self).__init__(*args, **kwargs)
        self.fields['id__in'] = forms.CharField(
            label=_("IDs List"),
            required=False,
            help_text="lista di ID separate da virgole"
        )
        self.fields['title__icontains'] = forms.CharField(label=_("Title"), required=False)
        self.fields['description__icontains'] = forms.CharField(label=_("Description"), required=False)
        self.fields['keywords__icontains'] = forms.CharField(label=_("Keywords"), required=False)
        self.fields['organization__name__icontains'] = forms.CharField(
            label=_("Organization"), required=False
        )
        self.fields['project__name__icontains'] = forms.CharField(
            label=_("Project"), required=False
        )
        self.fields['status'] = forms.ChoiceField(
            label=_('status'), choices=[("", "-"*9)] + list(STATUS_CODES),
            required=False
        )
        self.fields['project'] = forms.ModelChoiceField(
            TicketProject.objects.all(), label=_('Project'), required=False
        )
        self.fields['date_create__gte'] = JumboDateField(
            label=_("date create >="),
            required=False
        )
        self.fields['date_create__lte'] = JumboDateField(
            label=_("date create <="),
            required=False
        )
        self.fields['deadline__gte'] = JumboDateField(
            label=_("Deadline >="),
            required=False
        )
        self.fields['deadline__lte'] = JumboDateField(
            label=_("Deadline <="),
            required=False
        )
class AssignedToMultipleChoiceField(ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except:
            return "%s" % (obj.user)

class AssignedByChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except:
            return "%s" % (obj.user)

class TicketAdminForm(AdminModelForm):
    class Meta:
        model = Ticket

    class Media:
        js = ('ticket/js/ticket_form.js',)

    def __init__(self, *args, **kws):
        super(TicketAdminForm, self).__init__(*args, **kws)

        user = get_current_user()
        # for non-superuser
        try:
            ticket_user = user.ticket_user
        except:
            ticket_user = None

        # questo controllo serve per fare in modo che agli utenti guest non vengano
        # aggiunti i campi project e organization
        if not user.is_superuser and ticket_user and ticket_user.is_guest:
            org_project = TicketProject.objects.all()
            self.fields['project'] = forms.ModelChoiceField(queryset=org_project, label=_("Project"))
            return

        self.fields['project'] = ChainedChoiceField(
            model=TicketProject, label=_("Project")
        )
        self.fields['organization'] = AutocompleteModelChoiceField(
            'ticket_organization', label=_("Organization"),
            on_select_token='on_select_organization',
            on_delete_token='on_delete_organization',
        )

        self.fields['assigned_to'].required = True
        organization = self.get_object_id('organization')
        project = self.get_object_id('project')

        assigned_bys = TicketUser.objects.none()
        assigned_tos = TicketUser.objects.none()
        projects = TicketProject.objects.none()

        if organization:
            projects = TicketProject.objects.filter(
                (Q(date_end__gte=datetime.date.today())|Q(date_end__isnull=True)),
                date_start__lte=datetime.date.today(), organization__id=organization,
            )
        if project:
            ticket_project = TicketProject.objects.get(pk=project)
            assigned_bys = ticket_project.managers.filter(user__is_active=True).order_by('user__last_name')
            assigned_tos = ticket_project.staff.filter(user__is_active=True).order_by('user__last_name')

        # modifico tendina degli ASSIGNED_BY
        self.fields['assigned_by'] = AssignedByChoiceField(
            queryset=assigned_bys, required=False
        )

#        # modifico tendina degli ASSIGNED_TO
#        self.fields['assigned_to'].queryset=assigned_tos

        # modifico tendina degli ASSIGNED_TO
        self.fields['assigned_to'] = AssignedToMultipleChoiceField(
            queryset=assigned_tos,
            widget=FilteredSelectMultiple(
                verbose_name=_('Assigned to'),
                is_stacked=False
            )
        )

        # modifico tendina degli PROJECT
        self.fields['project'].queryset = projects

    def clean_assigned_by(self):
        # Se non viene messo l'assigned_by lo metto di default uguale al submitter del ticket
        # (solo se questo e' un TicketUser, altrimenti lascio None)
        #
        user = get_current_user()
        ticket_user = None
        try:
            ticket_user = user.ticket_user
        except:
            pass

        if not self.cleaned_data['assigned_by']:
            self.cleaned_data['assigned_by'] = ticket_user

        return self.cleaned_data['assigned_by']


    def _get_changed_data(self):
        if self._changed_data is None:
            self._changed_data = []
            # XXX: For now we're asking the individual widgets whether or not the
            # data has changed. It would probably be more efficient to hash the
            # initial data, store it in a hidden field, and compare a hash of the
            # submitted data, but we'd need a way to easily get the string value
            # for a given field. Right now, that logic is embedded in the render
            # method of each widget.
            for name, field in self.fields.items():
                prefixed_name = self.add_prefix(name)
                data_value = field.widget.value_from_datadict(self.data, self.files, prefixed_name)
                if not field.show_hidden_initial:
                    initial_value = self.initial.get(name, field.initial)
                else:
                    initial_prefixed_name = self.add_initial_prefix(name)
                    hidden_widget = field.hidden_widget()
                    initial_value = hidden_widget.value_from_datadict(
                        self.data, self.files, initial_prefixed_name)
                if field.widget._has_changed(initial_value, data_value):
                    self._changed_data.append(name)
        return self._changed_data
    changed_data = property(_get_changed_data)


class TicketAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("ticket/css/admin_ticket.css",)}

    save_on_top = True
    search_form = TicketAdminSearchForm
    form = TicketAdminForm
    default_group_by = 'project_groupby'
    inlines = [TicketDocumentInline,]

    fieldsets = (
        (_('General Information'), {
                'fields':
                    ('title', 'status', 'public',
                                ('priority', ), ('tipology', 'category',),
                                ('organization', 'project'),
                                ('assigned_by',),
                    'assigned_to', 'description', 'keywords', 'duration', 'deadline')}),
        (_('Additional Information'), {
            'classes': ('collapse',),
             'fields': (
                        ('note',),
                        ('date_create', 'date_last_modify','creator','last_modifier',)
                        )
        }),
    )
    guest_fieldsets = (
        (_('General Information'), {
                'fields':
                    ('title','project','tipology','priority','deadline','description')
                }
        ),
    )

#    filter_horizontal = ('assigned_to',)
    list_display = ('get_public', 'status', 'get_ticket_priority_icon', 'get_date_create', 'deadline', 'get_title',
                    'organization', 'get_assigned_to', 'get_num_assistances', 'get_num_comments',
                    'get_edit_icon','get_detail_icon', 'get_delete_icon')
    list_display_guest = ('get_date_create', 'get_ticket_status_icon', 'deadline', 'title', 'get_num_comments',
                    'get_edit_icon','get_detail_icon')
    list_display_csv = ('get_title', 'organization', 'description')
    list_editable = ['status']
    list_editable_guest = []
    list_per_page = 50

    def get_title(self, result):
        return """<a class="iframe" href='%(url)s?_popup=1&title=1&nobutton=1'">%(title)s</a>""" % {
            'url':'%s%s/detail/' % (reverse('admin:ticket_ticket_changelist'), result.id),
            'title':result.title,
        }
    get_title.allow_tags = True
    get_title.short_description = _("Title")

    def get_ticket_status_icon(self, result):
        return """<img src='%(url)sticket/img/ticket_status_%(status)s.png' alt='%(window_title)s' title='%(window_title)s'/>""" % {
            'url':settings.MEDIA_URL,
            'status':result.status,
            'window_title':ugt(result.get_status_display())
        }
    get_ticket_status_icon.allow_tags = True
    get_ticket_status_icon.short_description = _("ST")

    def get_ticket_priority_icon(self, result):
        return """<img src='%(url)sticket/img/ticket_priority_%(priority)s.png' alt='%(window_title)s' title='%(window_title)s'/>""" % {
            'url':settings.MEDIA_URL,
            'priority':result.priority,
            'window_title':ugt(result.get_priority_display())
        }
    get_ticket_priority_icon.allow_tags = True
    get_ticket_priority_icon.short_description = _("P")

    def get_deadline(self, result):
        if result.deadline:
            not_expired = "%(deadline)s" % {
                'deadline': date_format(result.deadline, 'DATE_FORMAT'),
            }
            expired = "<span class='object_error'>%(deadline)s</span>" % {
                'deadline': date_format(result.deadline, 'DATE_FORMAT'),
            }
            if result.deadline < datetime.date.today():
                return expired
            else:
                return not_expired
        else: return ""
    get_deadline.allow_tags = True
    get_deadline.short_description = _("Deadline")
        
    def response_add(self, request, obj, post_url='../', post_url_continue='../%s/', msg=None):
        """
        Determines the HttpResponse for the add_view stage.
        """
        post_url='../?status__in=1,2,3,6,7'
        if request.POST.has_key("_save_and_add_assistance"):
            post_url = "%s?ticket=%s" % (reverse('admin:ticket_ticketassistance_add'), obj.id)
        return super(TicketAdmin, self).response_add(request, obj, post_url, post_url_continue, msg)


    def response_change(self, request, obj, post_url='../', msg=None):
        """
        Determines the HttpResponse for the add_view stage.
        """
        post_url='../?status__in=1,2,3,6,7'
        if request.POST.has_key("_save_and_add_assistance"):
            post_url = "%s?ticket=%s" % (reverse('admin:ticket_ticketassistance_add'), obj.id)
        return super(TicketAdmin, self).response_change(request, obj, post_url, msg)

    def get_actions(self, request):
        actions = super(TicketAdmin, self).get_actions(request)
        if not request.user.is_superuser:
            try:
                ticket_user = request.user.ticket_user
            except:
                ticket_user = None
            if ticket_user and ticket_user.is_guest:
                del actions['set_active']
                del actions['set_disactive']

        return actions

    @csrf_protect_m
    def changelist_view(self, request, extra_context=None):
        # for non-superuser
        if not request.user.is_superuser:
            try:
                ticket_user = request.user.ticket_user
            except:
                ticket_user = None

            try:
                if ticket_user and ticket_user.is_guest:
                    self.list_display = ['action_checkbox'] + list(self.list_display_guest)
                    self.list_editable = list(self.list_editable_guest)
                response = super(TicketAdmin, self).changelist_view(request, extra_context)
            finally:
                original_list_display = TicketAdmin.list_display
                original_list_editable = TicketAdmin.list_editable
                if 'action_checkbox' not in original_list_display and self.actions is not None:
                    original_list_display = ['action_checkbox'] +  list(original_list_display)
                original_list_editable = list(original_list_editable)
                # Reset fieldsets to its original value
                self.list_editable = original_list_editable
                self.list_display = original_list_display
            return response
        else:
            return super(TicketAdmin, self).changelist_view(request, extra_context)


    @csrf_protect_m
    @transaction.commit_on_success
    def add_view(self, request, form_url='', extra_context=None):
        # for non-superuser
        try:
            ticket_user = request.user.ticket_user
        except:
            ticket_user = None

        if not request.user.is_superuser:
            try:
                if ticket_user.is_guest:
                    self.fieldsets = self.guest_fieldsets
                response = super(TicketAdmin, self).add_view(request, form_url, extra_context)
            finally:
                # Reset fieldsets to its original value
                self.fieldsets = TicketAdmin.fieldsets
            return response
        else:
            return super(TicketAdmin, self).add_view(request, form_url, extra_context)


    @csrf_protect_m
    @transaction.commit_on_success
    def change_view(self, request, object_id, extra_context=None):
        # Se il ticket e' chiuso non gli presento il form di modifica
        # ma lo riporto al dettaglio
        ticket = Ticket.objects.get(pk=object_id)
        if ticket.status == 0:
            return HttpResponseRedirect(reverse('admin:ticket_ticket_detail', object_id))

        # for non-superuser
        try:
            ticket_user = request.user.ticket_user
        except:
            ticket_user = None

        if not request.user.is_superuser:
            try:
                if ticket_user.is_guest:
                    self.fieldsets = self.guest_fieldsets
                response = super(TicketAdmin, self).change_view(request, object_id, extra_context)
            finally:
                # Reset fieldsets to its original value
                self.fieldsets = TicketAdmin.fieldsets
            return response
        else:
            return super(TicketAdmin, self).change_view(request, object_id, extra_context)


    def save_model(self, request, obj, form, change):
        if not change:
            # Salvo il submitter in automatico in base all'utente collegato
            obj.submitter = request.user

            # Per i ticket Guest setto in automatico i valori di anagrafica e is_public
            try:
                ticket_user = request.user.ticket_user
            except:
                ticket_user = None

            if ticket_user and ticket_user.is_guest:
                try:
                    organization = request.user.contact.organization
                    obj.public = True
                    obj.organization = organization
                except:
                    pass

        super(TicketAdmin, self).save_model(request, obj, form, change)

        send_mail = get_settings("ticket.settings", "SEND_MAIL")
        if send_mail:
            TicketEmail(request, obj, form, change).send()


    def project_groupby(self, result):
        return result.project
    project_groupby.short_description = _("Project")
    project_groupby.admin_order_field = ('project', 'project__organization')


    def get_public(self, result):
        return _boolean_icon(result.public)
    get_public.short_description = _("P")
    get_public.allow_tags = True


    def get_date_create(self, result):
        return date_format(result.date_create, 'DATE_FORMAT')
    get_date_create.short_description = _("Create")
    get_date_create.admin_order_field = 'date_create'


    def get_project(self, result):
        return result.project.name
    get_project.short_description = _("Project")


    def get_assigned_to(self, result):
        assigned_to = result.get_string_assigned_to()
        if assigned_to:
            if len(assigned_to) > 15:
                return "<span title='%s'>%s...</span>" % (assigned_to, assigned_to[:15])
            else:
                return assigned_to
        else:
            return ""
    get_assigned_to.short_description = _('Assigned To')
    get_assigned_to.allow_tags = True


    def get_num_assistances(self, result):
        num_assistances = TicketAssistance.ticket_assistances.filter(ticket=result).count()
        if num_assistances > 0:
            if result.status == 0:
                img = "<img src='%sadmin/images/yes.jpg' title=\"Ticket Chiuso\" />" % settings.MEDIA_URL
            else:
                img = """<a href='%(reverse_url)s?ticket=%(ticket_id)s' onclick="jshowAddAnotherPopup(this, '%(window_title)s', 650, 1000, 100, 200);return false;"><img src='%(url)sadmin/images/add_yellow.png' title="%(window_title)s" style="vertical-align:middle;"/></a>""" % {
                                                                 'reverse_url': reverse('admin:ticket_ticketassistance_add'),
                                                                 'window_title': ugt("Add Assistance"),
                                                                 'url': settings.MEDIA_URL,
                                                                 'ticket_id': result.id
                                                           }


            return """%(img)s&nbsp;&nbsp;<a class="iframe" href='%(reverse_url)s?_popup=1'">%(num_assistances)s</a>""" % {'reverse_url': reverse('admin:ticket_ticketassistances_list', result.id),
                                                                 'window_title': ugt("View Ticket Assistance"),
                                                                 'num_assistances': num_assistances,
                                                                 'img': img,
                                                                 'ticket_id': result.id
                                                             }
        else:
            if result.status == 0:
                return "<img src='%sadmin/images/yes.jpg' title=\"Ticket Chiuso\" />" % settings.MEDIA_URL
            else:
                return """<a href='%(reverse_url)s?ticket=%(ticket_id)s' onclick="jshowAddAnotherPopup(this, 'Aggiungi Assistenza', 650, 1000, 100, 200);return false;"><img src='%(url)sadmin/images/work.png' title="%(window_title)s" /></a>""" % {
                                                                'reverse_url': reverse('admin:ticket_ticketassistance_add'),
                                                                 'window_title': ugt("Add Ticket Assistance"),
                                                                 'url': settings.MEDIA_URL,
                                                                 'ticket_id': result.id
                                                            }
    get_num_assistances.short_description = 'A'
    get_num_assistances.allow_tags = True


    def get_num_comments(self, obj):
        n_comment = obj.get_comments_count()
        if n_comment:
            num_comment = n_comment
        else:
            num_comment = ""
        return """<a class="iframe" href="%(reverse_url)s?_popup=1&is_comment_list=1&nobuttons=1"><img src='%(media_url)sadmin/images/comment.png' title=\"Lista Commenti\" style=\"vertical-align:middle;\"/>
        <span style="position: relative; bottom: 16px; left: 3px; font-size: 9px;display: block;">%(num_comments)s</span></a>""" % {
                                                                'reverse_url': reverse('admin:ticket_ticket_detail', obj.id),
                                                                 'media_url': settings.MEDIA_URL,
                                                                 'num_comments': num_comment
                                                             }
    get_num_comments.short_description = 'C'
    get_num_comments.allow_tags = True


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(TicketAdmin, self).get_urls()

        my_urls = patterns('',
            url(r'^add_comment/(?P<id>\d+)/$'   , 'django.views.generic.simple.direct_to_template', {'template': 'add_comment.html'}
                                                , name='ticket_ticket_add_comment'),
            url(r'^reopen/(?P<id>\d+)/$'        , self.admin_site.admin_view(ticket_views.ticket_reopen)
                                                , name='ticket_ticket_reopen'),
            url(r'^(?P<id>\d+)/assistances/$'   , self.admin_site.admin_view(ticket_views.ticket_assistances_list)
                                                , name='ticket_ticketassistances_list'),
        )
        return my_urls + urls


#########################################
#              TIPOLOGY                 #
#########################################
class TipologyTranslationInline(admin.TabularInline):
    model = TipologyTranslation
    extra = 3
    fields = ('language', 'name',)


class TipologyAdmin(ExtendibleModelAdmin):
    list_display = ('get_status_icon','name','description',
                    'get_edit_icon','get_detail_icon', 'get_delete_icon')
    ordering = ('name',)
    search_fields = ['name']
    fieldsets = (
        (_('General Information'), {
                'fields': ('name', 'description',)
        }),
        (_('Additional Information'), {
            'classes': ('collapse',),
             'fields': (
                        ('date_create', 'date_last_modify','creator','last_modifier',)
                        )
        }),
    )
    inlines = [TipologyTranslationInline,]


#########################################
#              CATEGORY                 #
#########################################
class CategoryTranslationInline(admin.TabularInline):
    model = CategoryTranslation
    extra = 3
    fields = ('language', 'name',)


class CategoryAdmin(ExtendibleModelAdmin):
    list_display = ('get_status_icon','name','description',
                    'get_edit_icon','get_detail_icon', 'get_delete_icon')
    ordering = ('name',)
    search_fields = ['name']
    fieldsets = (
        (_('General Information'), {
                'fields': ('name', 'description',)
        }),
        (_('Additional Information'), {
            'classes': ('collapse',),
             'fields': (
                        ('date_create', 'date_last_modify','creator','last_modifier',)
                        )
        }),
    )
    inlines = [CategoryTranslationInline,]


#########################################
#              PROJECT                  #
#########################################
class TicketProjectAdminSearchForm(forms.ModelForm):
    class Meta:
        model = TicketProject
        display_fields = (('name__icontains','organization__name__icontains'),
                          ('description__icontains'),
                          ('date_start__gte', 'date_start__lte'),
                          ('date_end__gte','date_end__lte'),
                          ('managers','staff'))

    def __init__(self, *args, **kwargs):
        super(TicketProjectAdminSearchForm, self).__init__(*args, **kwargs)
        self.fields['name__icontains'] = forms.CharField(
            label=_("Name"), required=False
        )
        self.fields['organization__name__icontains'] = forms.CharField(
            label=_("Organization"), required=False
        )
        self.fields['description__icontains'] = forms.CharField(
            label=_("Description"), required=False
        )
        self.fields['date_start__gte'] = JumboDateField(
            label=_("Date start from"), required=False,
        )
        self.fields['date_start__lte'] = JumboDateField(
            label=_("Date start to"), required=False
        )
        self.fields['date_end__gte'] = JumboDateField(
            label=_("Date end from"), required=False
        )
        self.fields['date_end__lte'] = JumboDateField(
            label=_("Date end to"), required=False
        )

class ManagerMultipleChoiceField(ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except:
            return "%s" % (obj.user)

class StaffMultipleChoiceField(ModelMultipleChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except:
            return "%s" % (obj.user)

class TicketProjectAdminForm(AdminModelForm):
    project_ptr_id = forms.IntegerField(
        required=False,
        widget=forms.HiddenInput,
        label=""
    )
    class Meta:
        model = TicketProject
        fields = ('managers', 'staff', 'project_ptr', 'assistance_email', 'icon', 'default_for_organization')

    def __init__(self, *args, **kws):
        super(TicketProjectAdminForm, self).__init__(*args, **kws)

        managers = TicketUser.objects.filter(enabled=True, user__is_active=True).order_by('user__last_name')
        staff = TicketUser.objects.filter(enabled=True, user__is_active=True).order_by('user__last_name')

        self.fields['managers'] = ManagerMultipleChoiceField(
            queryset=managers,
            widget=FilteredSelectMultiple(
                verbose_name=_('Managers'),
                is_stacked=False
            )
        )

        self.fields['staff'] = StaffMultipleChoiceField(
            queryset=staff,
            widget=FilteredSelectMultiple(
                verbose_name=_('Staff'),
                is_stacked=False
            )
        )

    def save(self, *args, **kwargs):
        tp = super(TicketProjectAdminForm, self).save(commit=False)

        ## sto aggiungendo partendo da un progetto esistente
        if self.cleaned_data['project_ptr_id']:
            project_id = self.cleaned_data['project_ptr_id']
            tp.project_ptr_id = project_id
            ## Trovo il Progetto da cui si estende il TicketProject
            project = Project.objects.get(id=project_id)
            tp.project_ptr = project
        ## sto modificando un TicketProject esistente
        else:
            project = tp.project_ptr

        tp.date_create = project.date_create
        tp.creator = project.creator
        tp.status = project.status
        tp.o_number = project.o_number
        tp.c_number = project.c_number
        tp.o_date = project.o_date
        tp.c_date = project.c_date
        tp.accepted = project.accepted
        tp.name = project.name
        tp.organization = project.organization
        tp.description = project.description
        tp.date_start = project.date_start
        tp.date_end = project.date_end
        tp.save()
        return tp

class TicketProjectAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("ticket/css/admin_ticket.css",)}
        js = ("ticket/js/ticketproject_fancybox.js",)

    save_on_top = True
    form = TicketProjectAdminForm
    fieldsets = (
        (_('General Information'), {
            'fields': (
                'icon', 'managers', 'staff',
                ('default_for_organization', 'assistance_email',),
                'project_ptr_id'
            )
        }),
    )
    filter_horizontal = ('managers', 'staff')
    list_display = ('get_status_icon', 'date_create', 'name', 'organization',
                    'description', 'percentage_done', 'get_edit_icon','get_detail_icon', 'get_delete_icon')
    list_per_page = 30
    ordering = ('name',)
    search_fields = ['name']
    search_form = TicketProjectAdminSearchForm
    use_fancybox = True

#########################################
#              ASSISTANCE               #
#########################################
class TicketAssistanceAdminSearchForm(forms.ModelForm):
    class Meta:
        model = TicketAssistance
        display_fields = (
            ('ticket__id__in', 'ticket__title__icontains',),
            ('ticket__project', 'ticket__organization__name__icontains'),
            ('worker', 'procedure'),
            ('date_begin__year','search_in_all_years'),
            ('date__gte', 'date__lte'),
        )

    def __init__(self, *args, **kwargs):
        super(TicketAssistanceAdminSearchForm, self).__init__(*args, **kwargs)
        BOOLEAN = (('', '---'), ('1', _('Yes')), ('0', _('No')),)
        self.fields['ticket__id__in'] = forms.CharField(label=_("Ticket IDs List"), required=False)
        self.fields['ticket__title__icontains'] = forms.CharField(label=_("Ticket title"), required=False)
        self.fields['ticket__project'] = forms.ModelChoiceField(queryset=TicketProject.objects.all(), label=_("Project"), required=False)
        self.fields['ticket__organization__name__icontains'] = forms.CharField(label=_("Organization"), required=False)
        self.fields['date__gte'] = JumboDateField(label=_("Date from"), required=False)
        self.fields['date__lte'] = JumboDateField(label=_("Date to"), required=False)
        self.fields['search_in_all_years'] = forms.BooleanField(label=_("Search In All Years"), required=False)
        self.fields['date_begin__year'] = forms.CharField(label=_("Year"), required=False)
        self.fields['worker'] = forms.ModelChoiceField(queryset = TicketUser.objects.all(), label=_("Worker"), required=False)
        self.fields['procedure'] = forms.ChoiceField(label=_("Ordered"), choices=BOOLEAN, required=False)

    # Modifico il filtro di ricerca in modo da filtrare sempre in base all'anno corrente, tranne nel caso in cui
    # nei parametri di ricerca venga specificato un anno oppure venga flaggata la checkbox search_in_all_years
    def change_search_filter(self, lookup_params, qs):
        if not lookup_params.has_key('date_begin__year') and not lookup_params.has_key('search_in_all_years'):
            qs = qs.filter(date_begin__year = datetime.date.today().year)

        if lookup_params.has_key('date__gte'):
            date = lookup_params['date__gte'].split("-")
            try:
                fixed_datetime = datetime.datetime(int(date[0]), int(date[1]), int(date[2]))
                qs = qs.filter(date_begin__gte = fixed_datetime)
            except:
                raise IncorrectLookupParameters
            del lookup_params['date__gte']

        if lookup_params.has_key('date__lte'):
            date = lookup_params['date__lte'].split("-")
            try:
                fixed_datetime = datetime.datetime(int(date[0]), int(date[1]), int(date[2]),23,59)
                qs = qs.filter(date_begin__lte = fixed_datetime)
            except:
                raise IncorrectLookupParameters
            del lookup_params['date__lte']

        if lookup_params.has_key('search_in_all_years'):
            del lookup_params['search_in_all_years']
        return lookup_params, qs

class WorkerChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.user.contact.organization, obj.user.contact)
        except:
            return "%s" % (obj.user)

class TicketAssistanceAdminForm(AdminModelForm):
    class Meta:
        model = TicketAssistance

    def __init__(self, *args, **kws):
        super(TicketAssistanceAdminForm, self).__init__(*args, **kws)
        user = get_current_user()

        self.fields['ticket'] =  AutocompleteModelChoiceField('ticket_ticket', label=_("Ticket"))

        workers = TicketUser.objects.filter(user=user)
        if user.has_perm("ticket.change_all_ticketassistances"):
            workers = TicketUser.objects.filter(
                enabled = True,
                is_guest = False,
                user__is_active = True,
                user__contact__organization__organization_type__category='com'
            )

        self.fields['worker'] = WorkerChoiceField(
            queryset=workers, required=True,
        )

class TicketAssistanceAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("ticket/css/admin_ticket.css",)}
        js = ("ticket/js/ticketproject_fancybox.js",)

    list_per_page = 30
    fieldsets = (
        (_('General Information'), {
                'fields':
                    ('ticket', 'worker',
                        ('date_begin', 'date_end',),
                        'job_description',
                        ('work_duration', 'extra_duration', 'travel_duration',),
                        ('kilometers', 'carrying', 'overnight_stay', 'pay_exit',),
                        'procedure'
                    ,)
            }),
    )
    default_group_by = 'date_group_by'

    list_display = ('get_title', 'get_ticket_status_icon', 'worker', 'get_project',
                    'get_organization', 'date_begin', 'date_end', 'procedure',
                    'get_edit_icon','get_detail_icon', 'get_delete_icon')

    list_display_csv = (
        'id', 'worker', 'get_project', 'get_organization', 'get_ticket_title',
        'date_begin', 'date_end', 'get_real_duration_minutes', 'work_duration',
        'extra_duration', 'travel_duration'
    )

    actions = ['create_procedure', 'set_as_ordered']

    form = TicketAssistanceAdminForm
    search_form = TicketAssistanceAdminSearchForm

    def get_real_duration_minutes(self, obj):
        return obj.real_duration.seconds/60
    get_real_duration_minutes.short_description = _("Real Duration")

    def get_calculated_duration_minutes(self, obj):
        return obj.calculated_duration.seconds/60
    get_calculated_duration_minutes.short_description = _("Calculated Duration")

    def get_ticket_title(self, result):
        return result.ticket.title
    get_ticket_title.short_description = _("Title")

    def get_title(self, result):
        return """<a class="iframe" href='%(url)s?_popup=1&title=1&nobutton=1'" >%(title)s</a>""" % {
            'url':'%s%s/detail/' % (reverse('admin:ticket_ticket_changelist'), result.ticket.id),
            'title':result.ticket.title,
        }
    get_title.allow_tags = True
    get_title.short_description = _("Title")

    def get_ticket_status_icon(self, result):
        return """<img src='%(url)sticket/img/ticket_status_%(status)s.png' alt='%(window_title)s' title='%(window_title)s'/>""" % {
            'url':settings.MEDIA_URL,
            'status':result.ticket.status,
            'window_title':'Ticket %s' % ugt(result.ticket.get_status_display())
        }
    get_ticket_status_icon.allow_tags = True
    get_ticket_status_icon.short_description = _("ST")

    def get_ticket_assistance(self):
        return TicketAssistance.objects.all().order_by('worker','date_begin')

    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(TicketAssistanceAdmin, self).get_urls()

        my_urls = patterns('',
            url(r'^calendar/(?P<type_list>\w+)/(?P<day>\w*)$'   , 'jumbo.apps.calendar.views.calendar_list'
                                                    , {
                                                        'model': TicketAssistance,
                                                        'queryset_view': self.get_ticket_assistance,
                                                        'field_date_begin': 'date_begin',
                                                        'field_date_end': 'date_end',
                                                        'template_dir':'admin/ticket/ticketassistance/calendar'
                                                      }
                                                    , name='ticket_ticketassistance_calendar'),
        )
        return my_urls + urls


    def save_model(self, request, obj, form, change):
        # Salvo il worker in automatico in base all'utente collegato
        #obj.worker = request.user.ticket_user
        if request.POST.has_key("_save_and_close_ticket"):
            obj.ticket.status = 0
            obj.ticket.save()
            obj.ticket.send_close_mail()
        super(TicketAssistanceAdmin, self).save_model(request, obj, form, change)


    def response_add(self, request, obj, post_url_continue='../%s/'):
        """
        Determines the HttpResponse for the add_view stage.
        """
        opts = obj._meta
        pk_value = obj._get_pk_val()

        msg = _('The %(name)s "%(obj)s" was added successfully.') % {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj)}
        # Here, we distinguish between different save types by checking for
        # the presence of keys in request.POST.
        if request.POST.has_key("_continue"):
            self.message_user(request, msg + ' ' + _("You may edit it again below."))
            if request.POST.has_key("_popup"):
                post_url_continue += "?_popup=1"
            return HttpResponseRedirect(post_url_continue % pk_value)

        if request.POST.has_key("_popup"):
            return HttpResponse('<script type="text/javascript">window.close();opener.location.reload();</script>')
        elif request.POST.has_key("_addanother"):
            self.message_user(request, msg + ' ' + (_("You may add another %s below.") % force_unicode(opts.verbose_name)))
            return HttpResponseRedirect(request.path)
        else:
            self.message_user(request, msg)

            # Figure out where to redirect. If the user has change permission,
            # redirect to the change-list page for this object. Otherwise,
            # redirect to the admin index.
            if self.has_change_permission(request, None):
                post_url = '../'
            else:
                post_url = '../../../'
            return HttpResponseRedirect(post_url)


    def date_group_by(self, obj):
        return obj.date_begin.strftime("%d/%m/%Y")
    date_group_by.short_description = _('Date')
    date_group_by.admin_order_field = ('-date_begin', 'worker')


    def get_project(self, obj):
        return obj.ticket.project or ''
    get_project.short_description = _("Project")


    def get_organization(self, obj):
        organization = obj.ticket.organization
        if not organization and obj.ticket.project:
            organization = obj.ticket.project.organization
        return organization or ''
    get_organization.short_description = _("Organization")

    def set_as_ordered(self, request, queryset):
        """
        Set assistance as ordered
        """
        rows_updated = 0
        for obj in queryset:
            obj.procedure = True
            obj.save()
            rows_updated += 1

        if rows_updated == 0:
            messages.error(request, "No Assistance updated")
        else:
            if rows_updated == 1:
                message_bit = "1 Assistance was"
            else:
                message_bit = "%s Assistances were" % rows_updated
            messages.info(request, "%s successfully updated." % message_bit)

    def create_procedure(self, request, queryset):
        owner = None
        rows_updated = 0
        post = request.POST

        for obj in queryset.order_by('ticket__organization__name', 'ticket__project__name'):
            ## cambio intestatario -> nuova Procedure
            if obj.ticket.organization != owner:
                owner = obj.ticket.organization
                procedure, error = obj.create_or_append_procedure()

                if not procedure and error:
                    messages.error(request, error)
                    continue

                rows_updated += 1

            ## stesso intestatario -> vecchia Procedure
            else:
                res, error = obj.create_or_append_procedure(procedure=procedure)

                if not res and error:
                    messages.error(request, error)
                    continue

                rows_updated += 1

        if rows_updated == 0:
            messages.error(request, "No Procedure Row was created")
        else:
            if rows_updated == 1:
                message_bit = "1 Procedure Row was"
            else:
                message_bit = "%s Procedure Rows were" % rows_updated
            messages.info(request, "%s successfully created." % message_bit)

        return HttpResponseRedirect(request.META['HTTP_REFERER'])
    create_procedure.short_description = _("Create Procedure")

    #def get_status_ticket(self, obj):
    #    is_close = False
    #    if obj.ticket.status == 0:
    #        is_close = True
    #    return _boolean_icon(is_close)
    #get_status_ticket.short_description = "TS"
    #get_status_ticket.allow_tags = True


class TicketUserAdmin(ExtendibleModelAdmin):
    list_per_page = 30
    list_display = ('user', 'enabled', 'is_guest', 'get_edit_icon', 'get_delete_icon')
    ordering = ('user__username',)
    use_fancybox = True
    fieldsets = (
        (_('General Information'), {
                'fields':
                    ('user', 'enabled', 'is_guest',)
            }),
    )

admin.site.register(TicketUser, TicketUserAdmin)
admin.site.register(Ticket, TicketAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Tipology, TipologyAdmin)
admin.site.register(TicketProject, TicketProjectAdmin)
admin.site.register(TicketAssistance, TicketAssistanceAdmin)

def get_active_ticket(request, query):
    return Ticket.objects.filter(status__in=[1,2,3,6,7]).order_by("title")

def get_autocomplete_ticket_organization(request, query):
    results = []
    try:
        org_exact = Organization.objects.get(code=query)
        results.append([org_exact.id, "%s - %s" % (org_exact.code, org_exact.name)])
    except:
        pass

    q1 = Q(name__istartswith=query)
    q2 = Q(code__istartswith=query)
    for other_org in Organization.objects.exclude(code=query).filter(status=1).filter(q1|q2)[:10]:
        results.append([other_org.id, "%s - %s" % (other_org.code, other_org.name)])
    return results

autocomplete.register('ticket_ticket', model = Ticket, views = get_active_ticket,
                      search_fields=('title__icontains',) , limit=10)

autocomplete.register('ticket_organization', model = Organization, views = get_autocomplete_ticket_organization)
